@@ -2,7 +2,7 @@
                           application.cpp  -  description
                              -------------------
     begin                : Mon Jan 7 2002
-    copyright            : (C) 1999-2002 by Jacques Fortier & Brian Ashe
+    copyright            : (C) 1999-2002 by Brian Ashe & Jacques Fortier
     email                : gtkpool@seul.org
  ***************************************************************************/
 
@@ -20,7 +20,6 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
-#include <string.h>
 #include <gtk/gtk.h>
 #include "application.h"
 #include "sound.h"
@@ -36,11 +35,15 @@
 #include "ball.h"
 #include "indentify_ball.h"
 
-static GList *pixmaps_directories = NULL;
+#define PI 3.14159265
+
+static GList *load_directories = NULL;
 
 Application::Application(){
+		strcpy(s1, "0");
+		strcpy(s2, "0");
 		gap = 29;
-		width = 722 + gap * 2; height = 362 + gap * 2;
+		width = 722 + gap * 2; height = 361 + gap * 2;
 		table.width = width - (gap * 2);
 		table.height = height - (gap * 2);
 		table.x = 0 + gap;
@@ -48,12 +51,14 @@ Application::Application(){
 
 		FRICTION = .06;
 		BUMPER_DRAG = .8;
-		COLLIDE_DRAG = .95;
+		COLLIDE_DRAG = .7;
 
 		selected = NULL;
 
+		// Initialize colors
 		black  = new GdkColor;
 		red = new GdkColor;
+		yellow = new GdkColor;
 		table_color = new GdkColor;
 		message_colours[1] = new GdkColor;
 		message_colours[2] = new GdkColor;
@@ -78,12 +83,19 @@ Application::Application(){
 Application::~Application(){
    	delete black;
    	delete red;
+   	delete yellow;
    	delete table_color;
+   	delete message_colours[1];
+   	delete message_colours[2];
+   	delete message_colours[3];
+   	delete message_colours[4];
+   	delete message_colours[14];
    	delete[] table_pixmaps;
    	delete[] balls_pixmaps;
    	delete[] balls_pixmap_masks;
    	delete[] balls_big_pixmaps;
    	delete[] balls_big_pixmap_masks;
+   	delete thegame;
 }
 
 void Application::init()
@@ -107,15 +119,19 @@ void Application::init()
 	gdk_colormap_alloc_color(colormap, message_colours[3], FALSE, TRUE);
 	gdk_colormap_alloc_color(colormap, message_colours[4], FALSE, TRUE);
 	gdk_colormap_alloc_color(colormap, message_colours[14], FALSE, TRUE);
+	
+	// Set colors
 	gdk_color_black(colormap, black);
 	gdk_color_parse("Red", red);
 	gdk_colormap_alloc_color(colormap, red, FALSE, TRUE);
+	gdk_color_parse("Yellow", yellow);
+	gdk_colormap_alloc_color(colormap, yellow, FALSE, TRUE);
 	message_colours[0] = black;
 
 	// setup adjustments -- these are used in options dialog
 	// to control physics
 	friction_adjustment = gtk_adjustment_new(0.06, 0, 1, 0.01, 0.1, 0);
-	collide_drag_adjustment = gtk_adjustment_new(0.95, 0, 1, 0.01, 0.05, 0);
+	collide_drag_adjustment = gtk_adjustment_new(0.7, 0, 1, 0.01, 0.05, 0);
 	bumper_drag_adjustment = gtk_adjustment_new(0.8, 0.0, 1.5, 0.01, 0.1, 0);
 
 	resetTable();
@@ -138,6 +154,8 @@ void Application::destroy()
 	gdk_colormap_free_colors(colormap, message_colours[3], 1);
 	gdk_colormap_free_colors(colormap, message_colours[4], 1);
 	gdk_colormap_free_colors(colormap, message_colours[14], 1);
+	
+	close_sound_device();
 }
 
 void Application::updateBalls () {
@@ -191,24 +209,32 @@ reiterated until there are no more collisions left in the frame
 				// Put balls in the rack when sunk
 				if (cp.ball_num != CUE)
 				{
-					if (game_name == eight_ball)
+					switch (game_name)
 					{
-						if (cp.ball_num > 0 && cp.ball_num < 9)
-							sunk_balls1.push_back(bnc->ball_num);
-						if (cp.ball_num > 7 && cp.ball_num < 16)
-							sunk_balls2.push_back(bnc->ball_num);
-					} else
-					{
-						sunk_balls1.push_back(bnc->ball_num);
+						case eight_ball:
+						thegame->eight_ball_rules(cp, bnc);
+						break;
+						case nine_ball:
+						thegame->nine_ball_rules(cp, bnc);
+						break;
+						case rotation:
+						thegame->rotation_rules(cp, bnc);
+						break;
+						case no_rules:
+						thegame->player_1.sunkballs.push_back(bnc->ball_num);
+						thegame->first_ball_sunk = true;
+						strcpy(thegame->comment, "Nice Shot!\n");
+						break;
 					}
 					
 					last_sunk = cp.ball_num;
+				} else
+				{
+					thegame->scratched = true;
 				}
 
 				update_rack();
-				balls.erase(bnc); // get rid of it
-				if(cp.cue)
-					new_cue_ball();
+				balls.erase(std::vector<Ball>::iterator(bnc)); // get rid of it
 				sunk_tf = true;
 			}
 			else             // otherwise bounce it off the bumper
@@ -219,6 +245,69 @@ reiterated until there are no more collisions left in the frame
 		}
 		else if (md <= time  &&  cb1 != NULL)
 		{
+			Ball *hb;
+			if (cb1->ball_num > 0)
+				hb = cb1;
+			else
+				hb = cb2;
+				
+			if (!thegame->first_ball_hit && thegame->cat_is_set)
+			{
+				switch (game_name)
+				{
+					case eight_ball:
+						if (thegame->turn == tplayer1)
+						{
+							if (hb->category == thegame->player_1.category ||
+							 (thegame->player_1.sunkballs.size() == 7 && hb->is_eight))
+							{
+								thegame->first_ball_hit = true;
+								thegame->hit_wrong_ball = false;
+							} else
+							{
+								thegame->first_ball_hit = true;
+								thegame->hit_wrong_ball = true;
+							}
+						} else if (thegame->turn == tplayer2)
+						{
+							if (hb->category == thegame->player_2.category ||
+							 (thegame->player_2.sunkballs.size() == 7 && hb->is_eight))
+							{
+								thegame->first_ball_hit = true;
+								thegame->hit_wrong_ball = false;
+							} else
+							{
+								thegame->first_ball_hit = true;
+								thegame->hit_wrong_ball = true;
+							}
+						}						
+					break;
+					
+					case nine_ball:
+						if (hb->ball_num == low_ball())
+						{
+							thegame->first_ball_hit = true;
+							thegame->hit_wrong_ball = false;
+						} else
+						{
+							thegame->first_ball_hit = true;
+							thegame->hit_wrong_ball = true;
+						}
+					break;
+					
+					case rotation:
+						if (hb->ball_num == low_ball())
+						{
+							thegame->first_ball_hit = true;
+							thegame->hit_wrong_ball = false;
+						} else
+						{
+							thegame->first_ball_hit = true;
+							thegame->hit_wrong_ball = true;
+						}
+					break;
+				}
+			}
 			cb1->collide(cb2, collide_drag());
 			collide_tf = true;
 		}
@@ -227,6 +316,7 @@ reiterated until there are no more collisions left in the frame
 	} while (time > 0);
 	// Apply friction deceleration to all balls
 	for_each(balls.begin(), balls.end(), apply_friction(friction()));
+	
 	// redraw aimer if it selects a moving ball
 	if((hit_moving && selected != NULL) &&
 	   (selected->vel.dx != 0 || selected->vel.dy != 0))
@@ -234,29 +324,31 @@ reiterated until there are no more collisions left in the frame
 		update_dirty(int(shoot.x), int(selected->x), int(shoot.y), int(selected->y));
 	}
 	
-	// All balls have been sunk (restart)
+/*	// All balls have been sunk (restart)
 	if((balls.size() == 1 && balls.back().is_cue) || balls.size() == 0)
 	{
 		resetTable();
-	}
+	}*/
 }
 
-void Application::replaceBall () {
+void Application::replaceBall() {
 	double ox = (table.width * 0.75) + gap;
 	double oy = height * 0.5;
    	
-   	// Take the last one of the right rack
-   	if (last_sunk > 0 && last_sunk < 9)
-   		sunk_balls1.pop_back();
-   	if (last_sunk > 7 && last_sunk < 16)
-   		sunk_balls2.pop_back();
+   	// If 8-Ball take the last one of the right rack
+   	if (game_name == eight_ball)
+   	{
+	   	if (last_sunk > 0 && last_sunk < 9)
+   			thegame->player_1.sunkballs.pop_back();
+	   	if (last_sunk > 7 && last_sunk < 16)
+	   		thegame->player_1.sunkballs.pop_back();
+	}
    	// Put it on the table at the foul spot
    	if (last_sunk > 0 && last_sunk < 16)	
-		new_ball(ox, oy, last_sunk);
+		new_ball(ox, oy, last_sunk, other);
 	
    	// Reset so you can't put too many balls on the table
    	last_sunk = 0;
-	
 	udl = table.x; udr = table.x + table.width;
 	udt = table.y; udb = table.y + table.height;
 	// Reset the racks
@@ -266,57 +358,105 @@ void Application::replaceBall () {
 
 void Application::ball_in_hand()
 {				
-	vector<Ball>::iterator cb;
+	if (placing_cue)
+		return;
+	std::vector<Ball>::iterator cb;
 	cb = find_if(balls.begin(), balls.end(), indentify_ball());
-	balls.erase(cb); // remove current cue ball
-	update_dirty(table.x, table.x + table.width, table.y, table.y + table.height);
-	new_cue_ball();
+	if (cb->is_cue)
+	{
+		balls.erase(cb); // remove current cue ball
+		update_dirty(table.x, table.x + table.width, table.y, table.y + table.height);
+		new_cue_ball();
+	}
+}
+
+int Application::low_ball() {
+	// Get lowest ball number still on table
+	int low = 16;
+	int cur;
+	std::vector<Ball>::iterator bs;
+	for (bs=balls.begin(); bs != balls.end(); bs++)
+	{
+		cur = bs->ball_num;
+		if (cur > 0 && cur < 16 && cur < low)
+			low = cur;
+	}
+	return low;
 }
 
 void Application::resetTable () {
 	double ox = (table.width * 0.75) + gap;
 	double oy = height * 0.5;
-	double ofx = sqrt((BALL_SIZE * BALL_SIZE) - ((BALL_SIZE / 2) * (BALL_SIZE / 2))) + .3;
-	double ofy = BALL_SIZE / 2 + .3;
+	double ofx = sqrt((BALL_SIZE * BALL_SIZE) - ((BALL_SIZE / 2) * (BALL_SIZE / 2))) + 1;
+	double ofy = BALL_SIZE / 2 + 1;
 	
 	balls.erase(balls.begin(), balls.end());
-	sunk_balls1.erase(sunk_balls1.begin(), sunk_balls1.end());
-	sunk_balls2.erase(sunk_balls2.begin(), sunk_balls2.end());
-	if(game_name == eight_ball)
+	// Setup game
+	if (thegame)
+		delete thegame;
+		
+	thegame = new Game;
+	
+	if(game_name == eight_ball || game_name == no_rules)
 	{
 		// Standard Rack up
-		new_ball((table.width / 4) + gap - 1, oy, CUE, true);
-		new_ball(ox, oy, PURPLE, false, false, false, true);
-		new_ball(ox + ofx, oy - ofy, RED_S, false, false, false, false, true);
-		new_ball(ox + ofx, oy + ofy, BLUE, false, false, false, true);
-		new_ball(ox + ofx * 2, oy - ofy * 2, GREEN_S, false, false, false, false, true);
-		new_ball(ox + ofx * 2, oy, EIGHT, false, true);
-		new_ball(ox + ofx * 2, oy + ofy * 2, BROWN_S, false, false, false, false, true);
-		new_ball(ox + ofx * 3, oy - ofy * 3, ORANGE, false, false, false, true);
-		new_ball(ox + ofx * 3, oy - ofy, YELLOW_S, false, false, false, false, true);
-		new_ball(ox + ofx * 3, oy + ofy, RED, false, false, false, true);
-		new_ball(ox + ofx * 3, oy + ofy * 3, BLUE_S, false, false, false, false, true);
-		new_ball(ox + ofx * 4, oy - ofy * 4, GREEN, false, false, false, true);
-		new_ball(ox + ofx * 4, oy - ofy * 2, PURPLE_S, false, false, false, false, true);
-		new_ball(ox + ofx * 4, oy, BROWN, false, false, false, true);
-		new_ball(ox + ofx * 4, oy + ofy * 2, ORANGE_S, false, false, false, false, true);
-		new_ball(ox + ofx * 4, oy + ofy * 4, YELLOW, false, false, false, true);
+		new_ball((table.width / 4) + gap - 1, oy, CUE, other, true);
+		new_ball(ox, oy, PURPLE, solids, false, false, false, true);
+		new_ball(ox + ofx, oy - ofy, RED_S, stripes, false, false, false, false, true);
+		new_ball(ox + ofx, oy + ofy - .5, BLUE, solids, false, false, false, true);
+		new_ball(ox + (ofx * 2) + .4, oy - (ofy * 2) - .3, GREEN_S, stripes, false, false, false, false, true);
+		new_ball(ox + (ofx * 2), oy, EIGHT, other, false, true);
+		new_ball(ox + (ofx * 2), oy + (ofy * 2), BROWN_S, stripes, false, false, false, false, true);
+		new_ball(ox + (ofx * 3) + .3, oy - (ofy * 3) - .6, ORANGE, solids, false, false, false, true);
+		new_ball(ox + (ofx * 3), oy - ofy, YELLOW_S, stripes, false, false, false, false, true);
+		new_ball(ox + (ofx * 3), oy + ofy, RED, solids, false, false, false, true);
+		new_ball(ox + (ofx * 3) + .3, oy + (ofy * 3), BLUE_S, stripes, false, false, false, false, true);
+		new_ball(ox + (ofx * 4), oy - (ofy * 4) - .7, GREEN, solids, false, false, false, true);
+		new_ball(ox + (ofx * 4), oy - (ofy * 2), PURPLE_S, stripes, false, false, false, false, true);
+		new_ball(ox + (ofx * 4), oy, BROWN, solids, false, false, false, true);
+		new_ball(ox + (ofx * 4) + .3, oy + (ofy * 2), ORANGE_S, stripes, false, false, false, false, true);
+		new_ball(ox + (ofx * 4), oy + (ofy * 4), YELLOW, solids, false, false, false, true);
 	}
 	else if(game_name == nine_ball)
 	{
+		thegame->cat_is_set = true;		// Otherwise it will get confused
+		
 		// Rack up for Game of 9-ball
-		new_ball((table.width / 4) + gap, oy, CUE, true);
+		new_ball((table.width / 4) + gap, oy, CUE, other, true);
 		new_ball(ox, oy, YELLOW);
 		new_ball(ox + ofx, oy - ofy, BLUE);
-		new_ball(ox + ofx, oy + ofy + .02, GREEN);
-		new_ball(ox + ofx * 2 - .02, oy - ofy * 2 - .01, BROWN);
-		new_ball(ox + ofx * 2, oy, YELLOW_S, false, false, true);
-		new_ball(ox + ofx * 2, oy + ofy * 2, RED);
-		new_ball(ox + ofx * 3, oy - ofy, ORANGE);
-		new_ball(ox + ofx * 3, oy + ofy, EIGHT);
-		new_ball(ox + ofx * 4, oy, PURPLE);
+		new_ball(ox + ofx, oy + ofy + .4, GREEN);
+		new_ball(ox + (ofx * 2) - .6, oy - (ofy * 2) - .3, BROWN);
+		new_ball(ox + (ofx * 2), oy, YELLOW_S, other, false, false, true);
+		new_ball(ox + (ofx * 2) - .3, oy + (ofy * 2), RED);
+		new_ball(ox + (ofx * 3), oy - ofy, ORANGE);
+		new_ball(ox + (ofx * 3) - .4, oy + ofy - .7, EIGHT);
+		new_ball(ox + (ofx * 4), oy - .5, PURPLE);
+	} else if (game_name == rotation)
+	{
+		thegame->cat_is_set = true;		// Otherwise it will get confused
+		
+		// Rack up balls for Rotation
+		new_ball((table.width / 4) + gap - 1, oy, CUE, other, true);
+		new_ball(ox, oy, YELLOW);
+		new_ball(ox + ofx, oy - ofy, RED_S);
+		new_ball(ox + ofx, oy + ofy, ORANGE_S);
+		new_ball(ox + ofx * 2, oy - ofy * 2, PURPLE);
+		new_ball(ox + ofx * 2, oy, BROWN_S);
+		new_ball(ox + ofx * 2, oy + ofy * 2, GREEN);
+		new_ball(ox + ofx * 3, oy - ofy * 3, YELLOW_S);
+		new_ball(ox + ofx * 3, oy - ofy, EIGHT);
+		new_ball(ox + ofx * 3, oy + ofy, BLUE_S);
+		new_ball(ox + ofx * 3, oy + ofy * 3, BROWN);
+		new_ball(ox + ofx * 4, oy - ofy * 4, BLUE);
+		new_ball(ox + ofx * 4, oy - ofy * 2, PURPLE_S);
+		new_ball(ox + ofx * 4, oy, ORANGE);
+		new_ball(ox + ofx * 4, oy + ofy * 2, GREEN_S);
+		new_ball(ox + ofx * 4, oy + ofy * 4, RED);
 	}
+	
 	sunk_balls = 0;
+	last_sunk = 0;
 	udl = table.x; udr = table.x + table.width;
 	udt = table.y; udb = table.y + table.height;
 	update_rack();
@@ -368,34 +508,46 @@ void Application::paint (GdkDrawable *pixmap, GdkGC *gc) {
 		gdk_draw_line(pixmap, gc, int(stick_start.x), int(stick_start.y),
 		              int(shoot.x), int(shoot.y));
 		// Draw Aiming Line
-		stick.dx = -stick.dx;
-		stick.dy = -stick.dy;
-		double a = stick.mag() / 800;
-		stick.dx /= a; stick.dy /= a;
-		double sx = stick_start.x - stick.dx;
-		double sy = stick_start.y - stick.dy;
-
-		gdk_gc_set_foreground(gc, black);
-		gdk_gc_set_line_attributes(gc, 1, GDK_LINE_ON_OFF_DASH, GDK_CAP_BUTT, GDK_JOIN_MITER);
-		gdk_draw_line(pixmap, gc, int(stick_start.x), int(stick_start.y),
-		              int(sx), int(sy));
+		if (!aimlineoff)
+		{
+			stick.dx = -stick.dx;
+			stick.dy = -stick.dy;
+			double a = stick.mag() / 800;
+			stick.dx /= a; stick.dy /= a;
+			double sx = stick_start.x - stick.dx;
+			double sy = stick_start.y - stick.dy;
+	
+			gdk_gc_set_foreground(gc, black);
+			gdk_gc_set_line_attributes(gc, 1, GDK_LINE_ON_OFF_DASH, GDK_CAP_BUTT, GDK_JOIN_MITER);
+			gdk_draw_line(pixmap, gc, int(stick_start.x), int(stick_start.y),
+			              int(sx), int(sy));
+		}
 	}
 }
 
 void Application::mouse_down (double x, double y) {
-	vector<Ball>::iterator bb;
-	bb = find_if(balls.begin(), balls.end(), pointer_selects( x, y, hit_moving));
-	if(bb != balls.end())
-	{
-		selected = bb;
+	std::vector<Ball>::iterator bb;
+ 	bb = find_if(balls.begin(), balls.end(), pointer_selects( x, y, hit_moving));
+ 	if(bb != balls.end())
+ 	{
+#if _CPP_CSTDLIB == 1
+		// FIXME: horrible, non-portable, converting a vector iterator
+		// to a pointer using g++ 3.0 private interface :-(
+		// -- Philip Martin <philip_martin@ntlworld.com>
+		selected = bb.base();
+#else
+ 		selected = bb;
+#endif
 		shoot.x = x; shoot.y = y;
+		thegame->new_shot();
 	}
 }
 
 void Application::mouse_up (double x, double y) {
 	if (selected != NULL) {
 		selected->shoot(shoot);
-		update_dirty(table.x, table.x + table.width, table.y, table.y + table.height);
+		update_dirty(0, width, 0, height);
+		thegame->shot = true;
 	}
 	selected = NULL;
 }
@@ -405,22 +557,28 @@ void Application::mouse_drag (double x, double y) {
 		Vec2D vel;
 		double sx = double(selected->x), sy = double(selected->y);
 		double xdist = (sx - x), ydist = (sy - y);
-		vel.setVec( xdist, ydist );
-		if(superhuman || (vel.mag() <= 200))
-		{
-			shoot.x = x; shoot.y = y;
-		}
-		else
+		// decrease the following parameter for more accuracy
+		double dalpha = 0.00002;
+
+		double x1 = (xdist*cos(dalpha*(sx - x))), x2 = (xdist*sin(dalpha*(sx - x)));
+		double y1 = (ydist*cos(dalpha*(sy - y))), y2 = (ydist*sin(dalpha*(sy - y)));
+		xdist = (x1 * x2);
+		ydist = (y1 * y2);
+
+		vel.setVec(xdist, ydist);
+		
+		if(!(superhuman || (vel.mag() <= 200)))
 		{
 			// calculate a vector with the same direction but
-			// a magnitude of 150, since we don't want people
+			// a magnitude of 200, since we don't want people
 			// to be able to make superhuman shots
-			double k = vel.mag() / 200;
+			double k = (vel.mag() / 200);
 			vel.dx /= k; vel.dy /= k;
-			shoot.x = selected->x - vel.dx;
-			shoot.y = selected->y - vel.dy;
 		}
-		update_dirty(table.x, table.x + table.width, table.y, table.y + table.height);
+		shoot.x = selected->x - vel.dx;
+		shoot.y = selected->y - vel.dy;
+		
+		update_dirty(0, width, 0, height);
 	}
 }
 
@@ -452,9 +610,8 @@ void Application::new_cue_ball()
 	gdk_cursor_destroy(cursor);
 	gtk_signal_handler_block(GTK_OBJECT(drawing_area), button_press_handler_id);
 	selected = NULL; // just in case
-	gtk_signal_handler_unblock(GTK_OBJECT(drawing_area), qbp_hndlr);
-//	qbp_hndlr = gtk_signal_connect(GTK_OBJECT(drawing_area), "button_press_event",
-//	                               GTK_SIGNAL_FUNC(cue_ball_placed), this);
+	qbp_hndlr = gtk_signal_connect(GTK_OBJECT(drawing_area), "button_press_event",
+	                               GTK_SIGNAL_FUNC(cue_ball_placed), this);
 	placing_cue = true;
 }
 
@@ -463,19 +620,20 @@ gint Application::cue_ball_placed(GtkWidget *widget,
 								   Application *app)
 {
 	int x = int(event->x) ; int y = int(event->y);
-	vector<Ball>::iterator bb;
+	std::vector<Ball>::iterator bb;
 	bb = find_if(app->balls.begin(), app->balls.end(), pointer_selects(x, y, FALSE));
 	if (app->game_name == eight_ball || app->game_name == rotation)
 	{
 		if(x - BALL_SIZE / 2 > app->table.x &&
-		   x + BALL_SIZE / 2 < app->table.x + int(app->table.width / 4) + 8 &&
+//		   x + BALL_SIZE / 2 < app->table.x + int(app->table.width / 4) + 8 &&
+		   x + BALL_SIZE / 2 < app->table.x + app->table.width &&
 		   y - BALL_SIZE / 2 > app->table.y &&
 		   y + BALL_SIZE / 2 < app->table.y + app->table.height)
 		{
-			app->new_ball(x, y, CUE, true);
+			app->new_ball(x, y, CUE, other, true);
 			update_dirty(x - BALL_SIZE / 2, x + BALL_SIZE / 2, y - BALL_SIZE / 2,
 			             y + BALL_SIZE / 2);
-			gtk_signal_handler_block(GTK_OBJECT(app->drawing_area), app->qbp_hndlr);
+			gtk_signal_disconnect(GTK_OBJECT(app->drawing_area), app->qbp_hndlr);
 			gtk_signal_handler_unblock(GTK_OBJECT(app->drawing_area),
 			                           app->button_press_handler_id);
 			gdk_window_set_cursor(app->drawing_area->window, (GdkCursor *)NULL);
@@ -487,10 +645,10 @@ gint Application::cue_ball_placed(GtkWidget *widget,
 		   y - BALL_SIZE / 2 > app->table.y &&
 		   y + BALL_SIZE / 2 < app->table.y + app->table.height)
 		{
-			app->new_ball(x, y, CUE, true);
+			app->new_ball(x, y, CUE, other, true);
 			update_dirty(x - BALL_SIZE / 2, x + BALL_SIZE / 2, y - BALL_SIZE / 2,
 			             y + BALL_SIZE / 2);
-			gtk_signal_handler_block(GTK_OBJECT(app->drawing_area), app->qbp_hndlr);
+			gtk_signal_disconnect(GTK_OBJECT(app->drawing_area), app->qbp_hndlr);
 			gtk_signal_handler_unblock(GTK_OBJECT(app->drawing_area),
 			                           app->button_press_handler_id);
 			gdk_window_set_cursor(app->drawing_area->window, (GdkCursor *)NULL);
@@ -503,7 +661,7 @@ gint Application::cue_ball_placed(GtkWidget *widget,
 
 void Application::add_pixmap_directory (const gchar *directory)
 {
-  pixmaps_directories = g_list_prepend (pixmaps_directories, g_strdup (directory));
+  load_directories = g_list_prepend (load_directories, g_strdup (directory));
 }
 
 gchar *Application::check_file_exists (const gchar *directory, const gchar *filename)
@@ -531,7 +689,7 @@ GdkPixmap *Application::load_pixmap(GdkPixmap **mask, char *filename)
 	gchar *found_filename = NULL;
 
 	/* We first try any pixmaps directories set by the application. */
-	elem = pixmaps_directories;
+	elem = load_directories;
 	while (elem)
 	{
 		found_filename = check_file_exists ((gchar*)elem->data, filename);
@@ -599,10 +757,11 @@ void Application::load_ball_xpms()
 void Application::load_sounds()
 {
 	gchar *found_filename = NULL;
+	gchar *found_filename2 = NULL;
 	GList *elem;
-	char filename[200] = "click.raw";
-	coll_sndc = 1024; // max size to read
-	elem = pixmaps_directories;
+	char filename[20] = "ball_hit.raw";
+	coll_sndc = 12000; // max size to read
+	elem = load_directories;
 	while (elem)
 	{
 		found_filename = check_file_exists ((gchar*)elem->data, filename);
@@ -611,6 +770,21 @@ void Application::load_sounds()
 		elem = elem->next;
 	}
 	load_sound(found_filename, coll_snd, &coll_sndc);	
+	
+	/* really ugly way of loading the second sound
+	   make sure to fix this
+	*/
+	char filename2[20] = "ball_drop.raw";
+	pock_sndc = 194000; // max size to read
+	elem = load_directories;
+	while (elem)
+	{
+		found_filename2 = check_file_exists ((gchar*)elem->data, filename2);
+		if (found_filename2)
+		break;
+		elem = elem->next;
+	}
+	load_sound(found_filename2, pock_snd, &pock_sndc);	
 
 }
 
