@@ -2,7 +2,7 @@
                           check_pocket.h  -  description
                              -------------------
     begin                : Tue Jan 8 2002
-    copyright            : (C) 1999-2002 by Jacques Fortier & Brian Ashe
+    copyright            : (C) 1999-2002 by Brian Ashe & Jacques Fortier
     email                : gtkpool@seul.org
  ***************************************************************************/
 
@@ -18,18 +18,19 @@
 #ifndef CHECK_POCKET_H
 #define CHECK_POCKET_H
 #include "ball.h"
+#include "game.h"
 
 
 /**
-  *@author Brian Ashe
+  *@author Brian Ashe & Jacques Fortier
   */
 
 class check_pocket {
 public: 
 	int left, right, top, bottom, left_of_mid, right_of_mid;
 	double time;
-	int ball_num;
-	bool cue, eight;
+	int ball_num, category;
+	bool cue, eight, solid, stripe, nine;
 
 	check_pocket(GdkRectangle table, double ei);
 	~check_pocket();
