@@ -2,7 +2,7 @@
                           main.cpp  -  description
                              -------------------
     begin                : Mon Jan  7 12:56:44 EST 2002
-    copyright            : (C) 1999-2002 by Jacques Fortier & Brian Ashe
+    copyright            : (C) 1999-2002 by Brian Ashe & Jacques Fortier
     email                : gtkpool@seul.org
  ***************************************************************************/
 
@@ -19,6 +19,11 @@
 #include "application.h"
 #include "options.h"
 #include "sound.h"
+#include "game.h"
+
+/**
+  *@author Brian Ashe & Jacques Fortier
+  */
 
 // FPS (Frames Per Second) This sets the heartbeat of the program.
 const int FPS=64;
@@ -61,15 +66,14 @@ Application app;
 
 int main(int argc, char *argv[])
 {
-//	gboolean success[7];
-	GtkWidget *window;
-	GtkWidget *button;
-	GtkWidget *parent_box;
-	GtkWidget *top_box;
-	GtkWidget *bottom_box;
-	GtkWidget *entry;
+	GtkWidget *window;			// Main Window
+	GtkWidget *button;			// For buttons
+	GtkWidget *parent_box;		// Primary holder
+	GtkWidget *top_box;			// Where the buttons are
+	GtkWidget *bottom_box;		// Where the table and rack are
+	GtkWidget *entry;			// Unused. Where the dialog is
 
-	app.sound = true;
+	app.sound = true;			// Set sound to on
 
 	old_width = old_height = 0;
 
@@ -77,7 +81,7 @@ int main(int argc, char *argv[])
 
 	// setup main window
 	window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
-	gtk_window_set_title(GTK_WINDOW(window), "GtkPool");
+	gtk_window_set_title(GTK_WINDOW(window), "GTK Pool");
 	gtk_window_set_policy(GTK_WINDOW(window), FALSE, FALSE, FALSE);
 	gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER);
 	gtk_signal_connect(GTK_OBJECT(window), "destroy", GTK_SIGNAL_FUNC(destroy_event), NULL);
@@ -102,10 +106,6 @@ int main(int argc, char *argv[])
 	                   GTK_SIGNAL_FUNC(motion_notify_event), NULL);
 	app.button_press_handler_id = gtk_signal_connect(GTK_OBJECT(app.drawing_area),
 	          "button_press_event",GTK_SIGNAL_FUNC(button_press_event), NULL);
-	app.qbp_hndlr = gtk_signal_connect(GTK_OBJECT(app.drawing_area),
-	          "button_press_event",GTK_SIGNAL_FUNC(Application::cue_ball_placed),
-			&app);
-	gtk_signal_handler_block(GTK_OBJECT(app.drawing_area), app.qbp_hndlr);
 	gtk_signal_connect(GTK_OBJECT(app.drawing_area),"button_release_event",
 	                   GTK_SIGNAL_FUNC(button_release_event), NULL);
 	gtk_widget_set_events (app.drawing_area, GDK_EXPOSURE_MASK
@@ -144,12 +144,12 @@ int main(int argc, char *argv[])
 	                   GTK_SIGNAL_FUNC(restart_event), NULL);
 	gtk_box_pack_start(GTK_BOX(top_box), button, TRUE, TRUE, 0);
 	gtk_widget_show(button);
-	button = gtk_button_new_with_label("Replace");
+	button = gtk_button_new_with_label("Spot Last Ball");
 	gtk_signal_connect(GTK_OBJECT(button), "clicked",
 	                   GTK_SIGNAL_FUNC(replace_event), NULL);
 	gtk_box_pack_start(GTK_BOX(top_box), button, TRUE, TRUE, 0);
 	gtk_widget_show(button);
-	button = gtk_button_new_with_label("Set Cue Ball");
+	button = gtk_button_new_with_label("Ball in Hand");
 	gtk_signal_connect(GTK_OBJECT(button), "clicked",
 	                   GTK_SIGNAL_FUNC(set_cue), NULL);
 	gtk_box_pack_start(GTK_BOX(top_box), button, TRUE, TRUE, 0);
@@ -162,11 +162,11 @@ int main(int argc, char *argv[])
 	gtk_widget_show(button);*/
 
 	//setup chat box for multiplayer
-	app.chat_text = gtk_text_new(NULL, NULL);
-	gtk_widget_set_usize(app.chat_text, -2, 100);
+	app.chat_text = gtk_text_new((GtkAdjustment *)NULL, (GtkAdjustment *)NULL);
+	gtk_widget_set_usize(app.chat_text, -2, 60);
 	gtk_text_set_editable(GTK_TEXT(app.chat_text), FALSE);
 	gtk_text_set_word_wrap(GTK_TEXT(app.chat_text), TRUE);
-//	gtk_widget_show(app.chat_text);
+	gtk_widget_set_events (app.chat_text, GDK_NOTHING);
 //	networking not ready for this version
 
 	entry = gtk_entry_new();
@@ -181,7 +181,7 @@ int main(int argc, char *argv[])
 	gtk_box_pack_start(GTK_BOX(bottom_box), entry, TRUE, FALSE, 0);
 	gtk_widget_show(app.drawing_area);
 	gtk_widget_show(app.rack_area);
-
+	gtk_widget_show(app.chat_text);
 	gtk_widget_show(top_box);
 	gtk_widget_show(bottom_box);
 	gtk_widget_show(parent_box);
@@ -190,8 +190,9 @@ int main(int argc, char *argv[])
 
 	gtk_widget_show  (window);
 
-	app.print_message("gtkpool version 0.3.6, Copyright (C) 1999 Jacques Fortier");
-	app.print_message("This is free software, and you are welcome to redistribute it under certain conditions.  However, it comes with ABSOLUTELY NO WARRANTY; for details see the file 'COPYING'.");
+	app.print_message("GTKPool version 0.5.0, Copyright (C) 1999-2002 Brian Ashe & Jacques Fortier");
+	app.print_message("This is free software, and you are welcome to redistribute it under the terms of the GPL.");
+	app.print_message("However, it comes with ABSOLUTELY NO WARRANTY; for details see the file 'COPYING'.");
 
 	app.running = true;
 	gtk_timeout_add(1024 / FPS, GtkFunction(timeout_event), app.drawing_area);
@@ -330,6 +331,12 @@ gint timeout_event(gpointer widget)
 	if(!app.running)
 		return FALSE; // no more timeouts, no more animation
 	app.updateBalls();
+	if(app.collide_tf && app.sound)
+		play_sound(app.coll_snd, 2048);
+	if(app.sunk_tf && app.sound)
+		play_sound(app.pock_snd, 8192);
+	
+	// Redraw to clean up stuff on screen
 	if(udl != -1)
 	{
 		GdkRectangle ud_rect;
@@ -340,27 +347,124 @@ gint timeout_event(gpointer widget)
 		gdk_gc_unref(app.gc);
 		gtk_widget_draw(GTK_WIDGET(widget), &ud_rect);
 		udl = -1;
+	}	
+	
+	if (app.thegame->shot)
+	{
+		if (app.thegame->shot_over(app.balls))
+		{
+			if(app.thegame->switch_turns(app.game_name))
+		   		app.update_rack();
+		 	
+			if (app.thegame->scratched)
+   				app.new_cue_ball();
+
+			if (app.thegame->do_foul && !app.thegame->scratched)
+				app.ball_in_hand();
+				
+			// Replace the errant ball if playing rotation
+			if ((app.thegame->do_foul || app.thegame->scratched)
+				 && app.game_name == rotation
+				 && app.thegame->first_ball_sunk)
+			{
+				app.replaceBall();
+			}
+   	
+			// Give a comment on the shot and then reset for the next one.
+			app.print_message("");
+			app.print_message(app.thegame->comment);
+			app.thegame->shot = false;
+		}
 	}
-	if(app.collide_tf && app.sound)
-		play_sound(app.coll_snd, app.coll_sndc);
+	
 	return TRUE; // keep them timeouts coming
 }
 
 gint rack_expose_event (GtkWidget *widget, GdkEventExpose *event)
 {
 	int c = 0;
+	char p1_name [31];
+	char p2_name [31];
+	//GdkFont *fixed_font;
+	GdkFont *big_font;
+
+	/* Load a fixed font */
+	//fixed_font = gdk_font_load ("-misc-fixed-medium-r-*-*-*-140-*-*-*-*-*-*");
+	big_font = gdk_font_load ("-macromedia-Helmet-bold-r-normal-*-24-*-*-*-p-*-ascii-0");
+	if (big_font == NULL)
+		big_font = gdk_font_load ("-*-*-bold-r-normal-*-24-*-*-*-p-*-*");
+	
 	GdkGC *gc = gdk_gc_new(widget->window);
+	// Draw in our black background
 	gdk_gc_set_foreground(gc, app.black);
 	gdk_draw_rectangle(widget->window, gc, 1, event->area.x, event->area.y,
 	                   event->area.width, event->area.height);
 	gdk_gc_set_clip_mask(gc, app.rack_mask);
 	gdk_gc_set_clip_origin(gc, (app.width - 256) / 2, 3);
 
-	for(vector<int>::iterator i = app.sunk_balls1.begin();
-	    i < app.sunk_balls1.end(); i++)
+	// Set the color for each player by turn
+	switch (app.thegame->turn)
+	{
+		case tplayer1:
+			app.p1_color = app.red;
+			app.p2_color = app.yellow;
+			strcpy(p1_name, "*");
+			strcpy(p2_name, " ");
+			break;
+		
+		case tplayer2:
+			app.p1_color = app.yellow;
+			app.p2_color = app.red;
+			strcpy(p1_name, " ");
+			strcpy(p2_name, "*");
+			break;
+		
+	}
+	
+	// Show winner if there is one
+	if (app.thegame->winner >= 0)
+	{
+		gdk_gc_set_foreground(gc, app.red);
+		// See who won
+		if (app.thegame->winner == tplayer1)
+			gdk_draw_string(widget->window, big_font, gc, 23, 28, app.thegame->player_1.player_name);
+		if (app.thegame->winner == tplayer2)
+			gdk_draw_string(widget->window, big_font, gc, 23, 28, app.thegame->player_2.player_name);
+
+		gdk_draw_string(widget->window, big_font, gc, (app.width / 3), 28, "Is the WINNER!!!");
+		gdk_gc_unref(gc);
+		gdk_font_unref(big_font);
+		return FALSE;
+	}
+	
+	// Write the player names on the screen
+	if (app.game_name != no_rules)
+	{
+		strcat(p1_name, app.thegame->player_1.player_name);
+		strcat(p2_name, app.thegame->player_2.player_name);
+		gdk_gc_set_foreground(gc, app.p1_color);
+		gdk_draw_string(widget->window, big_font, gc, 23, 28, p1_name);
+		gdk_gc_set_foreground(gc, app.p2_color);
+		gdk_draw_string(widget->window, big_font, gc, 23, 64, p2_name);
+	
+	// Show scores in Rotation
+		if (app.game_name == rotation)
+		{
+			strcpy(app.s1, g_strdup_printf("-%d-", app.thegame->player_1.score));
+			strcpy(app.s2, g_strdup_printf("-%d-", app.thegame->player_2.score));
+			gdk_gc_set_foreground(gc, app.p1_color);
+			gdk_draw_string(widget->window, big_font, gc, (app.width - 80), 28, app.s1);
+			gdk_gc_set_foreground(gc, app.p2_color);
+			gdk_draw_string(widget->window, big_font, gc, (app.width - 80), 64, app.s2);
+		}	
+	}	
+	
+	// Show Player 1 balls sunk
+	for(std::vector<int>::iterator i = app.thegame->player_1.sunkballs.begin();
+	    i < app.thegame->player_1.sunkballs.end(); i++)
 	{
 		// draw each sunk ball onto the rack, in the order it was sunk
-		int rx = c * 36 + 3; // 32 pixels for the ball image, plus 4 for spacing, and a 3 px border
+		int rx = (app.width / 3) + (c * 36) + 3; // 32 pixels for the ball image, plus 4 for spacing, and a 3 px border
 		int ry = 3;
 		gdk_gc_set_clip_mask(gc, app.balls_big_pixmap_masks[*i]);
 		gdk_gc_set_clip_origin(gc, rx, ry);
@@ -368,11 +472,12 @@ gint rack_expose_event (GtkWidget *widget, GdkEventExpose *event)
 		c++;
 	}
 	c = 0;
-	for(vector<int>::iterator i = app.sunk_balls2.begin();
-	    i < app.sunk_balls2.end(); i++)
+	// Show Player 2 balls sunk
+	for(std::vector<int>::iterator i = app.thegame->player_2.sunkballs.begin();
+	    i < app.thegame->player_2.sunkballs.end(); i++)
 	{
 		// draw each sunk ball onto the rack, in the order it was sunk
-		int rx = c * 36 + 3; // 32 pixels for the ball image, plus 4 for spacing, and a 3 px border
+		int rx = (app.width / 3) + (c * 36) + 3; // 32 pixels for the ball image, plus 4 for spacing, and a 3 px border
 		int ry = 3 + 36; // second row
 		gdk_gc_set_clip_mask(gc, app.balls_big_pixmap_masks[*i]);
 		gdk_gc_set_clip_origin(gc, rx, ry);
@@ -381,6 +486,7 @@ gint rack_expose_event (GtkWidget *widget, GdkEventExpose *event)
 	}
 
 	gdk_gc_unref(gc);
+	gdk_font_unref(big_font);
 	return FALSE;
 }
 
@@ -494,7 +600,7 @@ void cd_close_event(GtkWidget *widget, GtkWidget *cd)
 void cd_destroy_event(GtkWidget *widget, Application *app)
 {
 	delete app->connect_dialog;
-	app->connect_dialog = NULL;
+	app->connect_dialog = (ConnectDialog *)NULL;
 }
 
 
