@@ -2,7 +2,7 @@
                           options.cpp  -  description
                              -------------------
     begin                : Tue Jan 8 2002
-    copyright            : (C) 1999-2002 by Jacques Fortier & Brian Ashe
+    copyright            : (C) 1999-2002 by Brian Ashe & Jacques Fortier
     email                : gtkpool@seul.org
  ***************************************************************************/
 
@@ -37,7 +37,7 @@ GtkWidget *open_opt_dlg(Application *app)
 	GtkWidget *hscale3;
 	GtkWidget *frame3;
 	GtkWidget *table3;
-	GSList *table3_group = NULL;
+	GSList *table3_group = (GSList *) NULL;
 	GtkWidget *radiobutton4;
 	GtkWidget *radiobutton5;
 	GtkWidget *radiobutton6;
@@ -48,12 +48,13 @@ GtkWidget *open_opt_dlg(Application *app)
 	GtkWidget *pixmap1;
 	GtkWidget *frame5;
 	GtkWidget *vbox1;
-	GSList *vbox1_group = NULL;
+	GSList *vbox1_group = (GSList *) NULL;
 	GtkWidget *radiobutton1;
 	GtkWidget *radiobutton2;
 	GtkWidget *radiobutton3;
 	GtkWidget *frame6;
 	GtkWidget *vbox2;
+	GtkWidget *checkbutton1;
 	GtkWidget *checkbutton2;
 	GtkWidget *checkbutton3;
 	GtkWidget *dialog_action_area1;
@@ -79,7 +80,7 @@ GtkWidget *open_opt_dlg(Application *app)
 	gtk_object_set_data (GTK_OBJECT (opt_dlg), "dialog_vbox1", dialog_vbox1);
 	gtk_widget_show (dialog_vbox1);
 
-	// Split sidways for the different options
+	// Split sideways for the different options
 	table1 = gtk_table_new (1, 5, FALSE);
 	gtk_widget_ref (table1);
 	gtk_object_set_data_full (GTK_OBJECT (opt_dlg), "table1", table1,
@@ -169,7 +170,6 @@ GtkWidget *open_opt_dlg(Application *app)
 	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
 	
 	// Select Pool Table
-	// Need to implement.
 	frame3 = gtk_frame_new (_("Table"));
 	gtk_widget_ref (frame3);
 	gtk_object_set_data_full (GTK_OBJECT (opt_dlg), "frame3", frame3,
@@ -321,6 +321,17 @@ GtkWidget *open_opt_dlg(Application *app)
 	gtk_widget_show (radiobutton3);
 	gtk_box_pack_start (GTK_BOX (vbox1), radiobutton3, TRUE, TRUE, 0);
 
+	// No Rules Button
+	radiobutton3 = gtk_radio_button_new_with_label (vbox1_group, _("No Rules"));
+	vbox1_group = gtk_radio_button_group (GTK_RADIO_BUTTON (radiobutton3));
+	gtk_widget_ref (radiobutton3);
+	gtk_object_set_data_full (GTK_OBJECT (opt_dlg), "radiobutton3", radiobutton3,
+	                          (GtkDestroyNotify) gtk_widget_unref);
+	gtk_signal_connect(GTK_OBJECT(radiobutton3), "toggled",
+	                   GTK_SIGNAL_FUNC(nr_toggled_event), app);
+	gtk_widget_show (radiobutton3);
+	gtk_box_pack_start (GTK_BOX (vbox1), radiobutton3, TRUE, TRUE, 0);
+
 	// Set Other Options
 	frame6 = gtk_frame_new (_("Other"));
 	gtk_widget_ref (frame6);
@@ -338,6 +349,26 @@ GtkWidget *open_opt_dlg(Application *app)
 	gtk_widget_show (vbox2);
 	gtk_container_add (GTK_CONTAINER (frame6), vbox2);
 
+	// Sound Off
+	checkbutton1 = gtk_check_button_new_with_label (_("No Sound"));
+	gtk_widget_ref (checkbutton1);
+	gtk_object_set_data_full (GTK_OBJECT (opt_dlg), "checkbutton1", checkbutton1,
+	                          (GtkDestroyNotify) gtk_widget_unref);
+	gtk_signal_connect(GTK_OBJECT(checkbutton1), "toggled",
+	                   GTK_SIGNAL_FUNC(sound_toggled_event), app);
+	gtk_widget_show (checkbutton1);
+	gtk_box_pack_start (GTK_BOX (vbox2), checkbutton1, TRUE, TRUE, 0);
+
+	// Aiming Line Off
+	checkbutton1 = gtk_check_button_new_with_label (_("No Aiming line"));
+	gtk_widget_ref (checkbutton1);
+	gtk_object_set_data_full (GTK_OBJECT (opt_dlg), "checkbutton1", checkbutton1,
+	                          (GtkDestroyNotify) gtk_widget_unref);
+	gtk_signal_connect(GTK_OBJECT(checkbutton1), "toggled",
+	                   GTK_SIGNAL_FUNC(aimline_toggled_event), app);
+	gtk_widget_show (checkbutton1);
+	gtk_box_pack_start (GTK_BOX (vbox2), checkbutton1, TRUE, TRUE, 0);
+
 	// Superhuman Shots
 	checkbutton2 = gtk_check_button_new_with_label (_("Superhuman Shots"));
 	gtk_widget_ref (checkbutton2);
@@ -382,6 +413,16 @@ void od_close_event(GtkWidget *widget, gpointer data)
 	gtk_widget_hide(GTK_WIDGET(data));
 }
 
+// Toggle Sound
+void sound_toggled_event(GtkWidget *widget, gpointer data)
+{
+	Application *app = (Application *)data;
+	if(GTK_TOGGLE_BUTTON(widget)->active)
+		app->sound = false;
+	else
+		app->sound = true;
+}
+
 // Toggle Hit Moving Balls
 void moving_toggled_event(GtkWidget *widget, gpointer data)
 {
@@ -392,6 +433,16 @@ void moving_toggled_event(GtkWidget *widget, gpointer data)
 		app->hit_moving = false;
 }
 
+// Toggle Superhuman
+void aimline_toggled_event(GtkWidget *widget, gpointer data)
+{
+	Application *app = (Application *)data;
+	if(GTK_TOGGLE_BUTTON(widget)->active)
+		app->aimlineoff = true;
+	else
+		app->aimlineoff = false;
+}
+
 // Toggle Superhuman
 void superhuman_toggled_event(GtkWidget *widget, gpointer data)
 {
@@ -424,6 +475,13 @@ void ro_toggled_event(GtkWidget *widget, Application *app)
 		app->resetTable();
 }
 
+void nr_toggled_event(GtkWidget *widget, Application *app)
+{
+	if(GTK_TOGGLE_BUTTON(widget)->active)
+		app->game_name = no_rules;
+		app->resetTable();
+}
+
 // Toggle Table
 void bp_table_toggled_event(GtkWidget *widget, Application *app)
 {
